#ifndef NASM_WARNINGS_H
#define NASM_WARNINGS_H

#ifndef WARN_SHR
# error "warnings.h should only be included from within error.h"
#endif

enum warn_index {
	WARN_IDX_NONE                    =   0, /* not suppressible */
	WARN_IDX_DB_EMPTY                =   1, /* no operand for data declaration */
	WARN_IDX_EA_ABSOLUTE             =   2, /* absolute address cannot be RIP-relative */
	WARN_IDX_EA_DISPSIZE             =   3, /* displacement size ignored on absolute address */
	WARN_IDX_ENVIRONMENT             =   4, /* nonexistent environment variable */
	WARN_IDX_FLOAT_DENORM            =   5, /* floating point denormal */
	WARN_IDX_FLOAT_OVERFLOW          =   6, /* floating point overflow */
	WARN_IDX_FLOAT_TOOLONG           =   7, /* too many digits in floating-point number */
	WARN_IDX_FLOAT_UNDERFLOW         =   8, /* floating point underflow */
	WARN_IDX_FORWARD                 =   9, /* forward reference may have unpredictable results */
	WARN_IDX_LABEL_ORPHAN            =  10, /* labels alone on lines without trailing `:' */
	WARN_IDX_LABEL_REDEF             =  11, /* label redefined to an identical value */
	WARN_IDX_LABEL_REDEF_LATE        =  12, /* label (re)defined during code generation */
	WARN_IDX_MACRO_DEF_CASE_SINGLE   =  13, /* single-line macro defined both case sensitive and insensitive */
	WARN_IDX_MACRO_DEF_GREEDY_SINGLE =  14, /* single-line macro */
	WARN_IDX_MACRO_DEF_PARAM_SINGLE  =  15, /* single-line macro defined with and without parameters */
	WARN_IDX_MACRO_DEFAULTS          =  16, /* macros with more default than optional parameters */
	WARN_IDX_MACRO_PARAMS_LEGACY     =  17, /* improperly calling multi-line macro for legacy support */
	WARN_IDX_MACRO_PARAMS_MULTI      =  18, /* multi-line macro calls with wrong parameter count */
	WARN_IDX_MACRO_PARAMS_SINGLE     =  19, /* single-line macro calls with wrong parameter count */
	WARN_IDX_NEGATIVE_REP            =  20, /* regative %rep count */
	WARN_IDX_NUMBER_OVERFLOW         =  21, /* numeric constant does not fit */
	WARN_IDX_OBSOLETE_NOP            =  22, /* instruction obsolete and is a noop on the target CPU */
	WARN_IDX_OBSOLETE_REMOVED        =  23, /* instruction obsolete and removed on the target CPU */
	WARN_IDX_OBSOLETE_VALID          =  24, /* instruction obsolete but valid on the target CPU */
	WARN_IDX_PHASE                   =  25, /* phase error during stabilization */
	WARN_IDX_PRAGMA_BAD              =  26, /* malformed %pragma */
	WARN_IDX_PRAGMA_EMPTY            =  27, /* empty %pragma directive */
	WARN_IDX_PRAGMA_NA               =  28, /* %pragma not applicable to this compilation */
	WARN_IDX_PRAGMA_UNKNOWN          =  29, /* unknown %pragma facility or directive */
	WARN_IDX_PREFIX_BND              =  30, /* invalid BND prefix */
	WARN_IDX_PREFIX_HLE              =  31, /* invalid HLE prefix */
	WARN_IDX_PREFIX_LOCK             =  32, /* LOCK prefix on unlockable instructions */
	WARN_IDX_PREFIX_OPSIZE           =  33, /* invalid operand size prefix */
	WARN_IDX_PREFIX_SEG              =  34, /* segment prefix ignored in 64-bit mode */
	WARN_IDX_PTR                     =  35, /* non-NASM keyword used in other assemblers */
	WARN_IDX_REGSIZE                 =  36, /* register size specification ignored */
	WARN_IDX_UNKNOWN_WARNING         =  37, /* unknown warning in -W/-w or warning directive */
	WARN_IDX_USER                    =  38, /* %warning directives */
	WARN_IDX_WARN_STACK_EMPTY        =  39, /* warning stack empty */
	WARN_IDX_ZEROING                 =  40, /* RESx in initialized section becomes zero */
	WARN_IDX_ZEXT_RELOC              =  41, /* relocation zero-extended to match output format */
	WARN_IDX_OTHER                   =  42, /* any warning not specifically mentioned above */
	WARN_IDX_ALL                     =  43  /* all possible warnings */
};

enum warn_const {
	WARN_NONE                        =   0 << WARN_SHR,
	WARN_DB_EMPTY                    =   1 << WARN_SHR,
	WARN_EA_ABSOLUTE                 =   2 << WARN_SHR,
	WARN_EA_DISPSIZE                 =   3 << WARN_SHR,
	WARN_ENVIRONMENT                 =   4 << WARN_SHR,
	WARN_FLOAT_DENORM                =   5 << WARN_SHR,
	WARN_FLOAT_OVERFLOW              =   6 << WARN_SHR,
	WARN_FLOAT_TOOLONG               =   7 << WARN_SHR,
	WARN_FLOAT_UNDERFLOW             =   8 << WARN_SHR,
	WARN_FORWARD                     =   9 << WARN_SHR,
	WARN_LABEL_ORPHAN                =  10 << WARN_SHR,
	WARN_LABEL_REDEF                 =  11 << WARN_SHR,
	WARN_LABEL_REDEF_LATE            =  12 << WARN_SHR,
	WARN_MACRO_DEF_CASE_SINGLE       =  13 << WARN_SHR,
	WARN_MACRO_DEF_GREEDY_SINGLE     =  14 << WARN_SHR,
	WARN_MACRO_DEF_PARAM_SINGLE      =  15 << WARN_SHR,
	WARN_MACRO_DEFAULTS              =  16 << WARN_SHR,
	WARN_MACRO_PARAMS_LEGACY         =  17 << WARN_SHR,
	WARN_MACRO_PARAMS_MULTI          =  18 << WARN_SHR,
	WARN_MACRO_PARAMS_SINGLE         =  19 << WARN_SHR,
	WARN_NEGATIVE_REP                =  20 << WARN_SHR,
	WARN_NUMBER_OVERFLOW             =  21 << WARN_SHR,
	WARN_OBSOLETE_NOP                =  22 << WARN_SHR,
	WARN_OBSOLETE_REMOVED            =  23 << WARN_SHR,
	WARN_OBSOLETE_VALID              =  24 << WARN_SHR,
	WARN_PHASE                       =  25 << WARN_SHR,
	WARN_PRAGMA_BAD                  =  26 << WARN_SHR,
	WARN_PRAGMA_EMPTY                =  27 << WARN_SHR,
	WARN_PRAGMA_NA                   =  28 << WARN_SHR,
	WARN_PRAGMA_UNKNOWN              =  29 << WARN_SHR,
	WARN_PREFIX_BND                  =  30 << WARN_SHR,
	WARN_PREFIX_HLE                  =  31 << WARN_SHR,
	WARN_PREFIX_LOCK                 =  32 << WARN_SHR,
	WARN_PREFIX_OPSIZE               =  33 << WARN_SHR,
	WARN_PREFIX_SEG                  =  34 << WARN_SHR,
	WARN_PTR                         =  35 << WARN_SHR,
	WARN_REGSIZE                     =  36 << WARN_SHR,
	WARN_UNKNOWN_WARNING             =  37 << WARN_SHR,
	WARN_USER                        =  38 << WARN_SHR,
	WARN_WARN_STACK_EMPTY            =  39 << WARN_SHR,
	WARN_ZEROING                     =  40 << WARN_SHR,
	WARN_ZEXT_RELOC                  =  41 << WARN_SHR,
	WARN_OTHER                       =  42 << WARN_SHR
};

struct warning_alias {
	const char *name;
	enum warn_index warning;
};

#define NUM_WARNING_ALIAS 50
extern const char * const warning_name[44];
extern const char * const warning_help[44];
extern const struct warning_alias warning_alias[NUM_WARNING_ALIAS];
extern const uint8_t warning_default[43];
extern uint8_t warning_state[43];

#endif /* NASM_WARNINGS_H */
